﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Catamaran
{
    /// <summary>
    /// Параметризованный набор объектов
    /// </summary>
    /// <typeparam name="T"></typeparam>
    internal class SetBoatsGeneric<T>
        where T : class
    {
        /// <summary>
        /// Массив объектов, которые храним
        /// </summary>
        private readonly List<T> _places;
        /// <summary>
        /// Количество объектов в массиве
        /// </summary>
        public int Count => _places.Count;

        private readonly int _maxCount;
        /// <summary>
        /// Конструктор
        /// </summary>
        /// <param name="count"></param>
        public SetBoatsGeneric(int count)
        {
            _maxCount = count;
            _places = new List<T>();
        }
        /// <summary>
        /// Добавление объекта в набор
        /// </summary>
        /// <param name="boat">Добавляемая лодка </param>
        /// <returns></returns>
        public int Insert(T boat)
        {

            for (int i = 0; i < _maxCount; i++)
            {
                if (i == Count)
                {
                    _places.Insert(i, boat);
                    return i;
                }
            }
            return -1;
        }
        /// <summary>
        /// Добавление объекта в набор на конкретную позицию
        /// </summary>
        /// <param name="boat">Добавляемая лодка</param>
        /// <param name="position">Позиция</param>
        /// <returns></returns>
        public int Insert(T boat, int position)
        {
            // TODO проверка позиции
            if (position < 0 || position >= _maxCount) return -1;
            // TODO проверка, что элемент массива по этой позиции пустой,если нет, то
            // проверка, что после вставляемого элемента в массиве есть пустой элемент
            // сдвиг всех объектов, находящихся справа от позиции до первого пустого элемента
            if (position == Count)
            {
                _places.Insert(position, boat);
                return position;
            }
            else
            {
                for (int i = position + 1; i < _maxCount; i++)
                {
                    if (i == Count)
                    {
                        for (int j = i - 1; j >= position; j--)
                        {
                            _places[j + 1] = _places[j];
                        }
                        _places.Insert(position, boat);
                        return position;
                    }
                }
                return -1;
            }
            // TODO вставка по позиции
        }
        /// <summary>
        /// Удаление объекта из набора с конкретной позиции
        /// </summary>
        /// <param name="position"></param>
        /// <returns></returns>
        public T Remove(int position)
        {
            // TODO проверка позиции
            if (position < 0 || position >= Count) return null;
            // TODO удаление объекта из массива, присовив элементу массива значение null
            var result = _places[position];
            _places.RemoveAt(position);
            return result;
        }
        /// <summary>
        /// Получение объекта из набора по позиции
        /// </summary>
        /// <param name="position"></param>
        /// <returns></returns>
        public T this[int position]
        {
            // TODO проверка позиции
            get
            {
                if (position >= 0 && position < _maxCount && position < Count)
                {
                    return _places[position];
                }
                else
                {
                    return null;
                }
            }
            set
            {
                if (position > 0 || position < Count)
                {
                    Insert(value, position);
                }
            }
        }
        /// <summary>
        /// Проход по набору до первого пустого
        /// </summary>
        /// <returns></returns>
        public IEnumerable<T> GetBoats()
        {
            foreach (var boat in _places)
            {
                if (boat != null)
                {
                    yield return boat;
                }
                else
                {
                    yield break;
                }
            }
        }

    }
}
